@@ -1,10 +1,11 @@
-MACS_VERSION = "2.0.9 20111102 (tag:alpha)"
-MACSDIFF_VERSION = "1.0.2 20111021 (tag:alpha)"
-FILTERDUP_VERSION = "1.0.0 20110906"
-RANDSAMPLE_VERSION = "1.0.0 20111102"
+MACS_VERSION = "2.1.0.20151222"
+#MACSDIFF_VERSION = "1.0.4 20110212 (tag:alpha)"
+FILTERDUP_VERSION = "1.0.0 20140616"
+RANDSAMPLE_VERSION = "1.0.0 20120703"
 MAX_PAIRNUM = 1000
 MAX_LAMBDA  = 100000
 FESTEP      = 20
+BUFFER_SIZE = 100000                   # np array will increase at step of 1 million items
 
 from array import array
 
@@ -31,4 +32,3 @@ elif array('d',[1]).itemsize == 4:
     FBYTE4 = 'd'
 else:
     raise Exception("FBYTE4 type cannot be determined!")
-
